/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "browseftpimpl.h"

BrowseFTPImpl::BrowseFTPImpl(OProfile *Profile_f, QString Filename, bool SelectDirectory_f) {
	setupUi(this);
	SetupCaptionsAndIcons();
	SelectDirectory = SelectDirectory_f;
	Profile = Profile_f;
	
	connect(&FTP, SIGNAL( listInfo(QUrlInfo) ), this, SLOT( FTPList(QUrlInfo) ));
	connect(LW_List, SIGNAL( itemDoubleClicked(QListWidgetItem*) ), this, SLOT( ListWidgedItemDoubleClicked(QListWidgetItem*) ));
	connect(LW_List, SIGNAL( itemClicked(QListWidgetItem*) ), this, SLOT( ListWidgedItemClicked(QListWidgetItem*) ));
	
	connect(PB_Back, SIGNAL( clicked() ), this, SLOT( PreviousButton() ));
	connect(PB_Forward, SIGNAL( clicked() ), this, SLOT( ForwardButton() ));
	connect(PB_Up, SIGNAL( clicked() ), this, SLOT( UpButton() ));
	connect(PB_Reload, SIGNAL( clicked() ), this, SLOT( List() ));
	connect(PB_NewDir, SIGNAL( clicked() ), this, SLOT( NewDirButton() ));
	
	connect(PB_Ok, SIGNAL( clicked() ), this, SLOT( OkButton() ));
	connect(PB_Cancel, SIGNAL( clicked() ), this, SLOT( reject() ));
	
	connect( &FTP, SIGNAL(commandFinished(int,bool)), SLOT(ftp_commandFinished()) );
    	connect( &FTP, SIGNAL(done(bool)), SLOT(ftp_done(bool)) );
	
	LE_Filename->setText(Filename);
	Current=Profile->ServerBaseDir;
	Connect(Profile);
	FTP.cd(Profile->ServerBaseDir);
	List();
	LE_Here->setText(Current);
	SetupButtons();
}

void BrowseFTPImpl::SetupCaptionsAndIcons() {
	this->setWindowIcon(QIcon(":/icons/oneclickftp.png"));
	
	PB_Back->setIcon(QIcon(":/icons/back.png"));
	PB_Forward->setIcon(QIcon(":/icons/forward.png"));
	PB_Up->setIcon(QIcon(":/icons/up.png"));
	PB_Reload->setIcon(QIcon(":/icons/reload.png"));
	PB_NewDir->setIcon(QIcon(":/icons/folder_new.png"));
	
	L_Here->setText(tr("Directory:"));
	L_Filename->setText(tr("Filename:"));
	L_Filetype->setText(tr("Files of Type:"));
	
	PB_Ok->setText(tr("OK"));
	PB_Ok->setIcon(QIcon(":/icons/ok.png"));
	PB_Cancel->setText(tr("Cancel"));
	PB_Cancel->setIcon(QIcon(":/icons/no.png"));
	
	PB_Back->setToolTip(tr("Back"));
	PB_Forward->setToolTip(tr("Forward"));
	PB_Up->setToolTip(tr("Up"));
	PB_Reload->setToolTip(tr("Reload"));
	PB_NewDir->setToolTip(tr("Create new directory"));
}

void BrowseFTPImpl::Connect(OProfile *Profile_f) {
	FTP.connectToHost(Profile_f->Server);
	FTP.login(Profile_f->Username, Profile_f->Password);
}

void BrowseFTPImpl::List() {
	pos=0;
	LW_List->clear();
	isDirectory.clear();
	
	if (!(Current == Profile->ServerBaseDir)) {
		LW_List->addItem("..");
		isDirectory.append(1);
		pos=1;	
	}
	
	FTP.list();
	setCursor(Qt::WaitCursor);
}

void BrowseFTPImpl::FTPList(QUrlInfo Url) {
	if (Url.isDir()) {
		QListWidgetItem *Item = new QListWidgetItem;
		Item->setText(Url.name());
		Item->setIcon(QIcon(":/icons/folder.png"));
		LW_List->insertItem(pos, Item);
		isDirectory.insert(pos, 1);
		pos++;
	} else if (!SelectDirectory) {
		QListWidgetItem *Item = new QListWidgetItem;
		Item->setText(Url.name());
		Item->setIcon(QIcon(":/icons/file.png"));
		LW_List->addItem(Item);
		isDirectory.append(0);
	}	
}

void BrowseFTPImpl::ListWidgedItemDoubleClicked(QListWidgetItem *item) {
	QString currentFile = item->text();
	if (isDirectory.at(LW_List->currentRow())) ChangeDir(currentFile, 0, 1);
	else {
		LE_Filename->setText(currentFile);
		PB_Ok->click();
	}
}

void BrowseFTPImpl::ListWidgedItemClicked(QListWidgetItem *item) {
	QString currentFile = item->text();
	if (!isDirectory.at(LW_List->currentRow())) LE_Filename->setText(currentFile);
	if (isDirectory.at(LW_List->currentRow()) && SelectDirectory) LE_Filename->setText(currentFile);
}

void BrowseFTPImpl::ChangeDir(QString Directory, bool Direct, bool History) {
	if (History) {
		Previous.append(Current);
		Forward.clear();	
	}
	if (!Direct) {
		if (Directory == "..") {
			if (!(Current == Profile->ServerBaseDir)) Current.truncate(Current.lastIndexOf("/", -3)+1);
		} else if (Directory == Profile->ServerBaseDir) {
			Current = Directory;	
		} else {		
			Current = Current+Directory+"/";
		}
	} else Current=Directory;
	
	LE_Here->setText(Current);
	SetupButtons();
	FTP.cd(Current);
	List();
}

void BrowseFTPImpl::PreviousButton() {
	QString Dir = Previous.last();
	Previous.removeAt(Previous.size()-1);
	Forward.append(Current);
	ChangeDir(Dir, 1, 0);
}

void BrowseFTPImpl::ForwardButton() {
	QString Dir = Forward.last();
	Forward.removeAt(Forward.size()-1);
	Previous.append(Current);
	ChangeDir(Dir, 1, 0);
}

void BrowseFTPImpl::UpButton() {
	ChangeDir("..", 0, 1);
}

void BrowseFTPImpl::SetupButtons() {
	if (Previous.size()==0) PB_Back->setEnabled(0); else PB_Back->setEnabled(1);
	if (Forward.size()==0) PB_Forward->setEnabled(0); else PB_Forward->setEnabled(1);
	if (Current == Profile->ServerBaseDir) PB_Up->setEnabled(0); else PB_Up->setEnabled(1);
}

void BrowseFTPImpl::OkButton() {
	if (SelectDirectory && (LW_List->currentRow() == -1)) LE_Filename->setText(Current);
	if (!LE_Filename->text().isEmpty()) {
		if ((LW_List->findItems(LE_Filename->text(), Qt::MatchFixedString).size()>0) && isDirectory.at(LW_List->row(LW_List->findItems(LE_Filename->text(), Qt::MatchFixedString).value(0))) && !SelectDirectory) {
			ChangeDir(LE_Filename->text(), 0, 1);
		} else if (SelectDirectory) {
			if (LW_List->currentRow() == -1) Result=Current;
				else Result=Current+LW_List->currentItem()->text()+"/";
				FTP.close();
				accept();
		} else {
			Result=Current+LE_Filename->text();
			Result.remove(0,1);
			FTP.close();
			accept();
		}		
	}
}

void BrowseFTPImpl::NewDirButton() {
	QString Dir = QInputDialog::getText(this, tr("Create new directory"), tr("Enter the name for the new directory"));
	if (!Dir.isEmpty()) FTP.mkdir(Dir);
	List();
}

void BrowseFTPImpl::ftp_commandFinished()
{
    delete FTP.currentDevice(); 
}

void BrowseFTPImpl::ftp_done( bool error )
{
    if ( error ) {
        QMessageBox::critical( this, tr("FTP Error"), FTP.errorString() );
        if ( FTP.state() == QFtp::Connected ) FTP.close();
    }
    setCursor(Qt::ArrowCursor);
}
