//
// C++ Implementation: fileinfodialogimpl
//
// Description: 
//
//
// Author: Stefan Lau <stefan.lau@gmx.net>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "fileinfodialogimpl.h"

FileInfoDialogImpl::FileInfoDialogImpl(OFileConnection *FileConnection, OProfile *Profile) {
	setupUi(this);
	
	SetupCaptionsAndIcons();

	L_FileLocalPath->setText(FileConnection->LocalFile);
	L_FileServerPath->setText(FileConnection->ServerFile);

	if (FileConnection->ExistsLocal) L_ExistsLocal->setText(tr("Yes")); else L_ExistsLocal->setText(tr("No"));
	if (FileConnection->ExistsServer) L_ExistsServer->setText(tr("Yes")); else L_ExistsServer->setText(tr("No"));

	if (FileConnection->ExistsLocal) L_SizeLocal->setText(SizeToString(FileConnection->SizeLocal)); else L_SizeLocal->setText("-");
	if (FileConnection->ExistsServer) L_SizeServer->setText(SizeToString(FileConnection->SizeServer)); else L_SizeServer->setText("-");

	QDateTime MDTML = QDateTime::fromString(FileConnection->LastModifiedLocal, "yyyyMMddhhmmss");
	QDateTime MDTMS = QDateTime::fromString(FileConnection->LastModifiedServer, "yyyyMMddhhmmss");
	MDTML = MDTML.addSecs(Profile->TimeDifference * 3600);
	if (FileConnection->ExistsLocal) L_LastModifiedLocal->setText(MDTML.toString("dd.MM.yyyy\nhh:mm")); else L_LastModifiedLocal->setText("-");
	if (FileConnection->ExistsServer) L_LastModifiedServer->setText(MDTMS.toString("dd.MM.yyyy\nhh:mm")); else L_LastModifiedServer->setText("-");
	


	connect(PB_OK, SIGNAL( clicked() ), this, SLOT( accept() ));
}


FileInfoDialogImpl::~FileInfoDialogImpl() {
}

void FileInfoDialogImpl::SetupCaptionsAndIcons() {
	this->setWindowTitle(tr("File Information"));

	PB_OK->setText(tr("OK"));
	PB_OK->setIcon(QIcon(":/icons/ok.png"));

	L_Image->setPixmap(QPixmap(":/icons/info.png").scaled(48, 48));
	L_FileInfo->setText(tr("File Info"));

	L_FileLocal->setText(tr("Local Filename:"));
	L_FileServer->setText(tr("Remote Filename:"));

	L_Local->setText(tr("Locally"));
	L_Server->setText(tr("Remote"));

	L_Exists->setText(tr("Exists"));
	L_Size->setText(tr("Size"));
	L_LastModified->setText(tr("Last Modified"));
}

QString FileInfoDialogImpl::SizeToString(int Size) {
	double SizeD;
	SizeD = Size;
	QString SizeE;
	if (SizeD >= 1024) {
		SizeD = SizeD / 1024;
		SizeE = "KB";
	}
	if (SizeD >= 1024) {
		SizeD = SizeD / 1024;
		SizeE = "MB";
	}
	if (SizeD >= 1024) {
		SizeD = SizeD / 1024;
		SizeE = "GB";
	}
	if (SizeD >= 1024) {
		SizeD = SizeD / 1024;
		SizeE = "TB";
	}
	return QString::number(SizeD, 'f', 2)+" " + SizeE;
}
