/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef __WIN32__
	#define Win 1
#else
	#define Win 0
#endif

#ifndef MAINWINDOWIMPL_H
#define MAINWINDOWIMPL_H

#include "def.h"

#include "ui_mainwindow.h"
#include <QFile>
#include <QDir>
#include <QSystemTrayIcon>
#include <QAction>
#include <QSettings>
#include <QTextStream>

class MainWindowImpl : public QMainWindow, public Ui::MainWindow
{
Q_OBJECT
public:
	MainWindowImpl( QWidget * parent = 0, Qt::WFlags f = 0 );
	
	QSystemTrayIcon TrayIcon;
	QDir Home;
	
	std::vector<OProfile*> Profiles;

	
	QTranslator *CurrentTrans;
	
	QSettings *Settings;

	QString Translator;

private:
	bool Changed;
	int ProfileChanged;
	OProfile *OldProfile;
	
public slots:
	bool CheckForPassword(bool Startup);
	void SetupCaptionsAndIcons();
	void SetupMenus();
	
	void ShowProfiles(int selected);
	void ShowProfileDetails(int selected);
	
	void SaveProfiles();
	void SaveProfilesDebug();
	
	void AboutApp();
	void ExitApp();
	
private slots:
	void ShowHide(QSystemTrayIcon::ActivationReason reason);
	
	void TW_ServerChanged(QTreeWidgetItem *Current, int Column);
	void TW_LocalChanged(QTreeWidgetItem *Current, int Column);
	void TW_Server_itemDoubleClicked(QTreeWidgetItem *Current, int Column);
	void TW_Local_itemDoubleClicked(QTreeWidgetItem *Current, int Column);
	void TW_Server_rightClick(const QPoint & iPoint);
	void TW_Local_rightClick(const QPoint & iPoint);
	
	void StartUpload(QAction *Profile);
	void StartDownload(QAction *Profile);
	
	void SettingsButton();
	
	void StartUploadButton();
	void StartDownloadButton();
	
	void AddProfileButton();
	void RemoveProfileButton();	
	void RenameProfileButton();
	void AddFileButton();
	void RemoveFileButton();
	void RenameLocalFile();
	void RenameServerFile();
	void ShowInfoBox();
	bool UpdateFileStatusButton();
	void BaseDirLocalSeekButton();
	void BaseDirServerSeekButton();
	
	void ExportProfilesButton();
	void ImportProfilesButton();
	bool Import03(QTextStream *InputStream, MainWindowImpl *win);
	bool Import05(QTextStream *InputStream, MainWindowImpl *win);
	
	void LWProfilesRowChanged(int NewRow);

	void ProfileDetailsChanged();
	void CB_CheckForMDTM_Changed(int ChangedTo);
	void CB_CheckForSize_Changed(int ChangedTo);
	void LE_Changed(QString dummy);
};
#endif
