/***************************************************************************
 *   Copyright (C) 2007 by Stefan Lau   *
 *   stefan.lau@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "settingsimpl.h"
#include <QTranslator>
#include <QtCrypto>
//
SettingsImpl::SettingsImpl( MainWindowImpl * parent, Qt::WFlags f, QSettings *Settings_f) 
	: QDialog(parent, f)
{
	setupUi(this);
	SetupCaptionsAndIcons();
	LW_List->setCurrentRow(0);
	Parent = parent;
	
	connect(PB_OK, SIGNAL( clicked() ), this, SLOT( OKButton() ));
	connect(PB_Cancel, SIGNAL( clicked() ), this, SLOT( CancelButton() ));
	connect(LW_List, SIGNAL( currentRowChanged(int) ), stackedWidget, SLOT( setCurrentIndex(int) ));
	connect(CB_Proxy, SIGNAL( stateChanged(int) ), this, SLOT( CBProxyChanged(int) ));
	connect(CB_PasswordOnStartup, SIGNAL( stateChanged(int) ), this, SLOT( CBPasswordChanged(int) ));
	connect(CB_PasswordOnTransfer, SIGNAL( stateChanged(int) ), this, SLOT( CBPasswordChanged(int) ));
	
	Settings = Settings_f;
	
	if (Settings->value("MinimizeOnStartup", 0).toBool()) CB_MinimizeOnStartup->setChecked(1);
	
	CB_Local->setCurrentIndex(Settings->value("Language", 0).toInt());
	
	if (Settings->value("Proxy/UseProxy", 0).toBool()) {
		CB_Proxy->setChecked(1);
		LE_ProxyIP->setText(Settings->value("Proxy/ProxyIP", "127.0.0.1").toString());
		LE_ProxyPort->setText(Settings->value("Proxy/ProxyPort", "80").toString());
	} else {
		LE_ProxyIP->setEnabled(0);
		LE_ProxyPort->setEnabled(0);
	}
	
	QCA::Initializer Init;
	QCA::Hash Hash("md5");
	if (!(Settings->value("EnterPasswordOnStartup", Hash.hash(QByteArray("Hell%No")).toByteArray()) == Hash.hash(QByteArray("Hell%No")).toByteArray()))
		CB_PasswordOnStartup->setChecked(1);
	if (!(Settings->value("EnterPasswordOnTransfer", Hash.hash(QByteArray("Not!At!All")).toByteArray()) == Hash.hash(QByteArray("Not!At!All")).toByteArray()))
		CB_PasswordOnTransfer->setChecked(1);
}

void SettingsImpl::SetupCaptionsAndIcons() {
	setWindowTitle(tr("Settings"));
	PB_OK->setText(tr("OK"));
	PB_OK->setIcon(QIcon(":/icons/ok.png"));
	PB_Cancel->setText(tr("Cancel"));
	PB_Cancel->setIcon(QIcon(":/icons/no.png"));
	
	LW_List->clear();
	
	QListWidgetItem *GeneralButton = new QListWidgetItem(LW_List);
	GeneralButton->setIcon(QIcon(":/icons/general.png"));
	GeneralButton->setText(tr("General"));
	GeneralButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	QListWidgetItem *LocalButton = new QListWidgetItem(LW_List);
	LocalButton->setIcon(QIcon(":/icons/local.png"));
	LocalButton->setText(tr("Translation"));
	LocalButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	QListWidgetItem *ProxyButton = new QListWidgetItem(LW_List);
	ProxyButton->setIcon(QIcon(":/icons/proxy.png"));
	ProxyButton->setText(tr("Proxy"));
	ProxyButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	QListWidgetItem *SecurityButton = new QListWidgetItem(LW_List);
	SecurityButton->setIcon(QIcon(":/icons/security.png"));
	SecurityButton->setText(tr("Security"));
	SecurityButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	GB_General->setTitle(tr("General"));
	CB_MinimizeOnStartup->setText(tr("Start in tray"));
	
	GB_Local->setTitle(tr("Translation"));
	L_Local->setText(tr("Select a Language"));
	CB_Local->addItem(tr("English"));
	CB_Local->addItem(tr("German"));
	
	GB_Proxy->setTitle(tr("Proxy settings"));
	CB_Proxy->setText(tr("Use Proxy"));	
	L_ProxyIP->setText(tr("IP / Adress"));
	L_ProxyPort->setText(tr("Port"));
	
	GB_Security->setTitle(tr("Security"));
	CB_PasswordOnStartup->setText(tr("Enter Password On Startup"));
	CB_PasswordOnTransfer->setText(tr("Enter Password On Transfer"));
	L_Password->setText(tr("Password:"));
}

void SettingsImpl::OKButton() {
	Settings->setValue("MinimizeOnStartup", CB_MinimizeOnStartup->checkState());
	Settings->setValue("Language", CB_Local->currentIndex());
	Settings->beginGroup("Proxy");
	Settings->setValue("UseProxy", CB_Proxy->checkState());
	Settings->setValue("ProxyIP", LE_ProxyIP->text());
	Settings->setValue("ProxyPort", LE_ProxyPort->text());
	Settings->endGroup();
	
	QCA::Initializer Init;
	if (CB_PasswordOnStartup->checkState() == Qt::Checked) {
		QCA::Hash Hash("md5");
		QByteArray Password;
		Password.append("Yes$Damit");
		Settings->setValue("EnterPasswordOnStartup", Hash.hash(Password).toByteArray());
		
		Password.clear();
		if (!LE_Password->text().isEmpty()) Password.append(LE_Password->text());
		Settings->setValue("Password", Hash.hash(Password).toByteArray());
	} else {
		QCA::Hash Hash("md5");
		QByteArray Password;
		Password.append("Hell%No");
		Settings->setValue("EnterPasswordOnStartup", Hash.hash(Password).toByteArray());
	}
	
	if (CB_PasswordOnTransfer->checkState() == Qt::Checked) {
		QCA::Hash Hash("md5");
		QByteArray Password;
		Password.append("OH()YEAH()");
		Settings->setValue("EnterPasswordOnTransfer", Hash.hash(Password).toByteArray());
		
		Password.clear();
		if (!LE_Password->text().isEmpty()) Password.append(LE_Password->text());
		Settings->setValue("Password", Hash.hash(Password).toByteArray());
	} else {
		QCA::Hash Hash("md5");
		QByteArray Password;
		Password.append("Not!At!All");
		Settings->setValue("EnterPasswordOnTransfer", Hash.hash(Password).toByteArray());
	}
	
	
	QTranslator Translator;
	QString TransFile;
	if (CB_Local->currentIndex() == 0) TransFile = "English.qm";
	if (CB_Local->currentIndex() == 1) TransFile = "German.qm";
	if (!Translator.load(":/tr/"+TransFile+".qm")) Translator.load(":/tr/English.qm");
	qApp->removeTranslator(Parent->CurrentTrans);
	Parent->CurrentTrans = &Translator;
	qApp->installTranslator(&Translator);
	
	Parent->SetupCaptionsAndIcons();
	Parent->SetupMenus();
	
	this->accept();
}

void SettingsImpl::CancelButton() {
	this->reject();
}

void SettingsImpl::CBProxyChanged(int State) {
	if (State == Qt::Checked) {
		LE_ProxyIP->setEnabled(1);
		LE_ProxyPort->setEnabled(1);
	} else {
		LE_ProxyIP->setEnabled(0);
		LE_ProxyPort->setEnabled(0);
	}
}

void SettingsImpl::CBPasswordChanged(int) {
	if (CB_PasswordOnStartup->checkState() == Qt::Checked) LE_Password->setEnabled(1); else
	if (CB_PasswordOnTransfer->checkState() == Qt::Checked) LE_Password->setEnabled(1); else LE_Password->setEnabled(0);
}
