#include "updatefilestatusimpl.h"

UpdateFileStatusDialogImpl::UpdateFileStatusDialogImpl(OProfile *Profile_f, QString ProxyIP, QString ProxyPort, std::vector<OFileConnection*> Files) {
	setupUi(this);
	
	FTP = new QFtp();
	Profile = Profile_f;
	SetupCaptionsAndIcons();

	if (Files.size() <= 0) FilesToProcess = Profile->Files; else FilesToProcess = Files;
	
	connect(PB_Cancel, SIGNAL( clicked() ), this, SLOT( reject() ));
	connect(PB_Cancel, SIGNAL( clicked() ), FTP, SLOT( abort() ));
	connect(PB_Cancel, SIGNAL( clicked() ), FTP, SLOT( close() ));

	connect(FTP, SIGNAL( rawCommandReply(int, QString) ), this, SLOT( rawCommandReply(int, QString) ));
	connect(FTP, SIGNAL( done(bool) ), this, SLOT( done(bool) ));

	show();

	L_FileStatusDialog->setText(tr("Gathering Information of Local Files"));
	LocalInformation();

	L_FileStatusDialog->setText(tr("Connecting to FTP server"));
	KeepOpen = 1;
	if ( FTP->state() != QFtp::Connected ) {
		if (ProxyIP != 0) {
			FTP->setProxy(ProxyIP, ProxyPort.toInt());
		}
		FTP->connectToHost(Profile->Server);
		FTP->login(Profile->Username, Profile->Password);
		KeepOpen = 0;
	}

	L_FileStatusDialog->setText(tr("Gathering Information of Remote Files"));
	ServerInformation();
	
	if (!KeepOpen) FTP->close();
	
	L_FileStatusDialog->setText(tr("Waiting for FTP server to finish"));
}

void UpdateFileStatusDialogImpl::SetupCaptionsAndIcons() {
	setWindowIcon(QIcon(":/icons/oneclickftp.png"));
	setWindowTitle(tr("Updating file status"));

	PB_Cancel->setText(tr("Cancel"));
	PB_Cancel->setIcon(QIcon(":/icons/no.png"));

	PB_Progress->setRange(0,-1);
}

void UpdateFileStatusDialogImpl::LocalInformation() {
	for (std::vector<OFileConnection*>::iterator i = FilesToProcess.begin(); i < FilesToProcess.end(); i++) {
		QFile *File = new QFile(Profile->LocalBaseDir + (*i)->LocalFile);
		if (File->exists()) {
			(*i)->ExistsLocal = 1;
			QDateTime MDTML = QFileInfo(Profile->LocalBaseDir + (*i)->LocalFile).lastModified();
			(*i)->LastModifiedLocal = MDTML.toString("yyyyMMddhhmmss");
			(*i)->LastModifiedLocal.chop(1);
			(*i)->LastModifiedLocal.append("0");
			(*i)->SizeLocal = QFileInfo(Profile->LocalBaseDir + (*i)->LocalFile).size();
		} else {
			(*i)->SizeLocal = -1;
			(*i)->LastModifiedLocal = QString("N/A");
			(*i)->ExistsLocal = 0;
		}
	}
}

void UpdateFileStatusDialogImpl::ServerInformation() {
	CurrentFile = -2;
	SizeFile = -1;
	
	FTP->rawCommand("TYPE I");

	QFile *File = new QFile(QDir::homePath()+"/.oneclickftp/oneclickftp.temp.gettimestamp");
	File->open(QIODevice::ReadWrite);
	File->resize(2);

	FTP->put(File->readAll(), "oneclickftp.temp.gettimestamp");
	FTP->rawCommand("MDTM oneclickftp.temp.gettimestamp");
	FTP->remove("oneclickftp.temp.gettimestamp");
	File->close();

	QDir(QDir::homePath()+"/.oneclickftp").remove("oneclickftp.temp.gettimestamp");

	FTP->cd(Profile->ServerBaseDir);
	for (std::vector<OFileConnection*>::iterator i = FilesToProcess.begin(); i < FilesToProcess.end(); i++) {
		QStringList dir;
		dir = (*i)->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd(dir.at(u));
		FTP->rawCommand("MDTM "+dir.last()); 
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
	}

	for (std::vector<OFileConnection*>::iterator i = FilesToProcess.begin(); i < FilesToProcess.end(); i++) {
		QStringList dir;
		dir = (*i)->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd(dir.at(u));
		FTP->rawCommand("SIZE "+dir.last()); 
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
		(*i)->New = 0;
	}
}

void UpdateFileStatusDialogImpl::rawCommandReply(int ReplyCode, QString Reply) {
	//QMessageBox::information(this, QString::number(ReplyCode), Reply);
	FileNotFound = 0;
	if (ReplyCode == 213 || ReplyCode == 550)
		{if (CurrentFile < (signed)FilesToProcess.size()-1) CurrentFile ++; else SizeFile++;}
	
	if (CurrentFile == -1) {
			Reply.chop(4);Reply.remove(0,6);
			QString DayStr = Reply;
			QString HourStr = Reply;
			DayStr.chop(2);HourStr.remove(0,2);
			int DayS = DayStr.toInt();int HourS = HourStr.toInt();

			QDateTime LocalTime = QDateTime::currentDateTime();
			int DayL = LocalTime.date().day();int HourL = LocalTime.time().hour();

			if (DayS == DayL - 1) HourL = HourL + 24;
			if (DayL == DayS - 1) HourS = HourS + 24;
			Profile->TimeDifference = HourS - HourL;
		} else {

		if (ReplyCode == 213 && SizeFile < 0) {
			FilesToProcess[CurrentFile]->ExistsServer = 1;
			FilesToProcess[CurrentFile]->LastModifiedServer = Reply;
			FilesToProcess[CurrentFile]->LastModifiedServer.chop(1);
			FilesToProcess[CurrentFile]->LastModifiedServer.append("0");
			L_FileStatusDialog->setText(FilesToProcess[CurrentFile]->ServerFile + tr(" exists on server."));
		}
		if (ReplyCode == 213 && SizeFile >= 0 && SizeFile < (signed)FilesToProcess.size()){
			FilesToProcess[SizeFile]->SizeServer = Reply.toInt();
			L_FileStatusDialog->setText(FilesToProcess[SizeFile]->ServerFile + tr(" has size: ") + Reply);
		}
		if (ReplyCode == 550) {
			if (SizeFile == -1) {
				FilesToProcess[CurrentFile]->ExistsServer = 0;
				FilesToProcess[CurrentFile]->LastModifiedServer = "N/A";
				L_FileStatusDialog->setText(FilesToProcess[CurrentFile]->ServerFile + tr(" does not exist on server."));
			} else {
				FilesToProcess[SizeFile]->SizeServer = -1;
				L_FileStatusDialog->setText(FilesToProcess[SizeFile]->ServerFile + tr(" does not exist on server."));
			}
			FileNotFound = 1;
		}
	}
}

void UpdateFileStatusDialogImpl::GoOn() {
	if (SizeFile == -1) {
		QStringList dir;
		dir = FilesToProcess[CurrentFile]->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
	} else {
		QStringList dir;
		dir = FilesToProcess[SizeFile]->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
	}

	for (int i = CurrentFile + 1; i < (signed)FilesToProcess.size();i++) {
		QStringList dir;
		dir = FilesToProcess[i]->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd(dir.at(u));
		FTP->rawCommand("MDTM "+dir.last()); 
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
	}

	for (int i = SizeFile + 1; i < (signed)FilesToProcess.size();i++) {
		QStringList dir;
		dir = FilesToProcess[i]->ServerFile.split("/");
		for (int u=0; u<dir.size()-1;u++) FTP->cd(dir.at(u));
		FTP->rawCommand("SIZE "+dir.last()); 
		for (int u=0; u<dir.size()-1;u++) FTP->cd("..");
	}

	

	if (SizeFile+1 >= (signed)FilesToProcess.size()) {
		FileNotFound = 0;
	}
}

void UpdateFileStatusDialogImpl::done( bool Error)
{
		if (Error && FileNotFound && (SizeFile+1 <= (signed)FilesToProcess.size())) GoOn(); else
		{
			if (Error) {
				for (int i = 0; i < (signed)FilesToProcess.size();i++) {
					FilesToProcess[i]->ExistsServer = 0;
				}
				QMessageBox::critical(this, "FTP Error", FTP->errorString());
				Accepted = 0;
			} else Accepted = 1;
			if ( FTP->state() == QFtp::Connected ) FTP->close();
			this->accept();
		}
}

void UpdateFileStatusDialogImpl::PB_Cancel_clicked() {
	this->setEnabled(0);

	FileNotFound = 0;
	FTP->abort();
	Accepted = 0;
	this->reject();
}
